require "spec_helper"
require "unit/external/example_source/processes/create_world"
require "unit/external/process_interface"
require "stuka/external/action_finder"

require "unit/external/example_source/actions/world/gather_materials"
require "unit/external/example_source/actions/world/scrap_world"
require "unit/external/example_source/actions/world/seperate_light_from_darkness"

describe Stuka::External::ActionFinder do

  let(:finder) { Stuka::External::ActionFinder.new }
  let(:process) { double() }

  before :each do
    Stuka::StukaPaths::ACTIONS_SOURCE_PATH.replace "#{File.dirname(__FILE__)}/example_source/actions"
  end

  it "finds actions" do
    gather_materials = finder.find_action("world::gather_materials", process)
    expect(gather_materials.class).to be(World::GatherMaterials)
    expect(gather_materials.process).to be(process)

    scrap_world = finder.find_action("world::scrap_world", process)
    expect(scrap_world.class).to be(World::ScrapWorld)
    expect(scrap_world.process).to be(process)

    seperate_light_from_darkness = finder.find_action("world::seperate_light_from_darkness", process)
    expect(seperate_light_from_darkness.class).to be(World::SeperateLightFromDarkness)
    expect(seperate_light_from_darkness.process).to be(process)
  end

  it "throws an UnknownAction error if it can't find the action class" do
    expect { finder.find_action("some::unknown_action", process) }.to raise_error(Stuka::External::UnknownAction)
  end

end
