require "spec_helper"
require "unit/external/example_source/create_world"
require "unit/external/process_interface"
require "stuka/external/business_process"

require "unit/external/example_source/world/gather_materials"
require "unit/external/example_source/world/scrap_world"
require "unit/external/example_source/world/seperate_light_from_darkness"

describe CreateWorld do

  it_behaves_like "a process"

  let(:stream) { double(puts: nil) }
  let(:param2) { double(param_method: nil) }

  let(:gather_materials)              { double(process: process) }
  let(:seperate_light_from_darkness)  { double(process: process) }
  let(:scrap_world)                   { double(process: process) }


  let(:process) do
  
    CreateWorld.new() do |on|
    
      on.success {
        puts "yay success"
      }
      on.failure {
        puts "nay failure"
      }

    end

  end

  before :each do

    allow(gather_materials).to receive(:run) { process.transition({stream: stream, param2: param2}, :success) }
    allow(seperate_light_from_darkness).to receive(:run) { process.transition({stream: stream}, :failure) }
    allow(scrap_world).to receive(:run) { process.transition({stream: stream}, :success) }

  end

  it "starts the process with BusinessProcess#run" do
    
    expect(World::GatherMaterials).to receive(:new).and_return(gather_materials)
    expect(World::SeperateLightFromDarkness).to receive(:new).and_return(seperate_light_from_darkness)
    expect(World::ScrapWorld).to receive(:new).and_return(scrap_world)

    expect(gather_materials).to receive(:run).with({stream: stream, param2: param2})
    expect(seperate_light_from_darkness).to receive(:run).with({stream: stream, param2: param2})
    expect(scrap_world).to receive(:run).with({stream: stream})

    process.run({stream: stream, param2: param2})

    expect(process.current_state).to eq Stuka::BusinessProcess::PROCESS_STATES[:finished]

  end

end
