require "spec_helper"
require "unit/external/example_source/processes/kill_monster"
require "unit/external/process_interface"
require "stuka/external/business_process"

describe KillMonster do

  it_behaves_like "a process"

  let(:finder) { double() }

  let(:process) do
  
    KillMonster.new(action_finder: finder)

  end
  
  let(:train_hard) { double(process: process) }
  let(:get_a_sharp_sword) { double(process: process) }
  let(:get_a_new_gun) { double(process: process) }
  let(:train_fast_running) { double(process: process) }
  let(:get_bullets) { double(process: process) }

  before :each do
    Stuka::StukaPaths::PROCESSES_SOURCE_PATH.replace "#{File.dirname(__FILE__)}/example_source"

    allow(finder).to receive(:find_action) do |action, process|
      
      case action
      when "monster::train_hard"
        train_hard
      when "monster::get_a_sharp_sword"
        get_a_sharp_sword
      when "monster::get_a_new_gun" 
        get_a_new_gun
      when "monster::train_fast_running" 
        train_fast_running
      when "gun::get_bullets"
        get_bullets
      end

    end

  end

  context "first path, killing monster with sword" do

    before :each do
      allow(train_hard).to receive(:run)          { self.process.transition({}, :good_with_sword) }
      allow(get_a_sharp_sword).to receive(:run)   { self.process.transition({}, :success) }
      allow(get_a_new_gun).to receive(:run)       { self.process.transition({}, :success) }
      allow(train_fast_running).to receive(:run)  { self.process.transition({}, :success) }
      allow(get_bullets).to receive(:run)         { self.process.transition({}, :success) }
    end

    it "sends the find_action message to finder so it can find action classes" do

      expect(finder).to receive(:find_action).with("monster::train_hard", process)
      expect(finder).to receive(:find_action).with("monster::get_a_sharp_sword", process)
      
      expect(finder).to_not receive(:find_action).with("monster::get_a_new_gun", process)
      expect(finder).to_not receive(:find_action).with("monster::get_bullets", process)

      process.run({}) do |on|
      
        on.success {
          puts "yay success"
        }
        on.failure {
          puts "nay failure"
        }

      end
    
      expect(process.current_state).to eq Stuka::External::BusinessProcess::PROCESS_STATES[:finished]

    end

  end

  context "second path, killing monster with firearm" do

    before :each do

      allow(train_hard).to receive(:run)          { self.process.transition({}, :sharp_shooter) }
      allow(get_a_sharp_sword).to receive(:run)   { self.process.transition({}, :success) }
      allow(get_a_new_gun).to receive(:run)       { self.process.transition({}, :success) }
      allow(train_fast_running).to receive(:run)  { self.process.transition({}, :success) }
      allow(get_bullets).to receive(:run)         { self.process.transition({}, :success) }

    end

    it "sends correct find_action messages to action finder object" do

      expect(finder).to receive(:find_action).with("monster::train_hard", process)
      expect(finder).to receive(:find_action).with("monster::get_a_new_gun", process)
      expect(finder).to receive(:find_action).with("gun::get_bullets", process)
      
      expect(finder).to_not receive(:find_action).with("monster::get_a_sharp_sword", process)

      process.run({}) do |on|
      
        on.success {
          puts "yay success"
        }
        on.failure {
          puts "nay failure"
        }

      end
    
      expect(process.current_state).to eq Stuka::External::BusinessProcess::PROCESS_STATES[:finished]

    end


  end

  context "failure path" do

    before :each do

      allow(train_hard).to receive(:run)          { self.process.transition({}, :good_with_sword) }
      allow(get_a_sharp_sword).to receive(:run)   { self.process.transition({}, :failure) }
      allow(get_a_new_gun).to receive(:run)       { self.process.transition({}, :success) }
      allow(train_fast_running).to receive(:run)  { self.process.transition({}, :success) }
      allow(get_bullets).to receive(:run)         { self.process.transition({}, :success) }
    
    end

    it "sends correct find_action messages to action finder object" do

      expect(finder).to receive(:find_action).with("monster::train_hard", process)
      expect(finder).to receive(:find_action).with("monster::get_a_sharp_sword", process)

      expect(finder).to_not receive(:find_action).with("monster::get_a_new_gun", process)
      expect(finder).to_not receive(:find_action).with("gun::get_bullets", process)
      
      process.run({}) do |on|
    
        on.success {
          puts "yay success"
        }
        on.failure {
          puts "nay failure"
        }

      end
      
      expect(process.current_state).to eq Stuka::External::BusinessProcess::PROCESS_STATES[:finished]

    end

  end

end
