require "spec_helper"
require "stuka/internal/source_builders/action_builder"
require "stuka/config/stuka_paths"
require "fileutils"

describe "ActionBuilder" do

  let(:stream) { double(puts: "") }

  around(:each) do |example|
    FileUtils.rm_rf("#{Stuka::StukaPaths::TEST_ACTIONS_SOURCE_PATH}/.", secure: true)
    example.run
    FileUtils.rm_rf("#{Stuka::StukaPaths::TEST_ACTIONS_SOURCE_PATH}/.", secure: true)
  end

  it "builds ruby source with ActionBuilder::build" do
    Stuka::ActionBuilder.build(stream)

    source_files = Dir["#{Stuka::StukaPaths.actions_source_path}/*/*.rb"]
    puts Dir["#{Stuka::StukaPaths.processes_definition_path}/**/*.rb"]
    source_files = source_files.map{ |path| File.basename(path) }
    expect(source_files.count).to eq(2)
    expect(source_files.include? "acc3.rb").to be(true)
    expect(source_files.include? "acc4.rb").to be(true)
  end

  it "notifies if the action file already exists using stdout passed as an argument" do

    Stuka::ActionBuilder.build(stream)

    expect(stream).to receive(:puts).with("action file bla::acc3 already exists.")

    Stuka::ActionBuilder.build(stream)

    source_files = Dir["#{Stuka::StukaPaths.actions_source_path}/*/*.rb"]
    source_files = source_files.map{ |path| File.basename(path) }
    expect(source_files.count).to eq(2)
    expect(source_files.include? "acc3.rb").to be(true)
    expect(source_files.include? "acc4.rb").to be(true)

  end

  it "edits only comments of existing action file and leaves the whole class intact" do

    make_test_action_file

    Stuka::ActionBuilder.build(stream)

    require_relative "#{File.dirname(__FILE__)}/actions_source_test/bla/acc3.rb"
    expect(Bla::Acc3.instance_methods.include? :additional_method).to be true
  end

  private

  def make_test_action_file
    file_content = <<-TEST

##
# example action file
# this is the old description
# it should be overriden
# by the new description
# of the action
# bla
#

# but the next comment section should stay

require 'stuka/external/business_action'

module Bla
  class Acc3 < Stuka::External::BusinessAction

    def run(params)
      params[stream].puts "test"
    end

    def additional_method

    end

  end
end

TEST

    FileUtils.mkdir_p "#{File.dirname(__FILE__)}/actions_source_test/bla"
    out_file = File.new("#{File.dirname(__FILE__)}/actions_source_test/bla/acc3.rb", "w")
    out_file.puts(file_content)
    out_file.close
  end

end