require "spec_helper"
require "stuka/internal/domain_builders/action_domain_builder"

describe Stuka::ActionDomainBuilder do
  
  let(:adb) { Stuka::ActionDomainBuilder.new }

  it "Creates an action and stores it in @instances attribute with ActionDomainBuilder#action method, this method is called in the dsl." do

    adb.action("sample::new_action1") do
      description "desc1"
      endings [:first_ending, :second_ending]
    end

    adb.action("sample::new_action2") do
      description "desc2"
      endings ["success", "failure"]
    end

    expect(adb.instances.count).to eq(2)
    
    act1 = adb.instances[0]
    expect(act1.namespace).to eq("sample")
    expect(act1.name).to eq("new_action1")
    expect(act1.description).to eq("desc1")
    expect(act1.endings).to eq([:first_ending, :second_ending])
  
    act2 = adb.instances[1]
    expect(act2.namespace).to eq("sample")
    expect(act2.name).to eq("new_action2")
    expect(act2.description).to eq("desc2")
    expect(act2.endings).to eq([:success, :failure])
  end

  context "when reserved 'prospect' namespace is used for an action" do

    it "should reaise an error with message" do
    
      dsl = Proc.new do
        description "this action uses reserved keywords"
        namespace "process"
      end
      
      expect { adb.action("new_action", &dsl) }.to raise_error "'process' namespace is reserved"

    end

  end

end