require "spec_helper"
require "stuka/internal/source_builders/process_builder"
require "stuka/config/stuka_paths"


describe "ProcessBuilder" do

  around(:each) do |example|
    FileUtils.rm_rf("#{Stuka::StukaPaths::TEST_PROCESSES_SOURCE_PATH}/.", secure: true)
    FileUtils.rm_rf("#{Stuka::StukaPaths::TEST_ACTIONS_SOURCE_PATH}/.", secure: true)
    example.run
    FileUtils.rm_rf("#{Stuka::StukaPaths::TEST_PROCESSES_SOURCE_PATH}/.", secure: true)
    FileUtils.rm_rf("#{Stuka::StukaPaths::TEST_ACTIONS_SOURCE_PATH}/.", secure: true)
  end

  it "builds ruby source with ProcessBuilder::build" do
    Stuka::ProcessBuilder.build

    process_source_files = Dir["#{Stuka::StukaPaths.processes_source_path}/*.rb"]
    process_source_files = process_source_files.map{ |path| File.basename(path) }
    expect(process_source_files.count).to eq(2)
    expect(process_source_files.include? "check_smth.rb").to be(true)
    expect(process_source_files.include? "test2.rb").to be(true)

    require_relative "processes_source_test/check_smth"

    process = Processes::CheckSmth.new

    expect(process.blocks).to eq(
      {
        "test_block_1" => {
          action: "bla::acc1",
          success: "test_block_2",
          failure: "process::failure",
        },

        "test_block_2" => {
          action: "bla::acc2",
          success: "process::success",
          failure: "process::failure",
        },
      
      }
    )

    expect(process.start_block).to eq("test_block_1")
  
  end

  it "builds action source files that are defined inside blocks" do
    Stuka::ProcessBuilder.build

    action_source_files = Dir["#{Stuka::StukaPaths.actions_source_path}/**/*.rb"]
    action_source_files = action_source_files.map{ |path| File.basename(path) }
    expect(action_source_files.count).to eq(2)
    expect(action_source_files.include? "acc1.rb").to be(true)
    expect(action_source_files.include? "acc2.rb").to be(true)
  end

end