require "spec_helper"
require "stuka/internal/domain_builders/process_domain_builder"

describe Stuka::ProcessDomainBuilder do
  
  it "creates a process, initializes it and stores it in @instances attribute with ProcessDomainBuilder#process, this method is called in the dsl." do
    pdb = Stuka::ProcessDomainBuilder.new

    pdb.process("new_process1") do
      description "desc1"
    end

    pdb.process("new_process2") do
      description "desc2"
    end

    expect(pdb.process_instances.count).to eq(2)

    proc1 = pdb.process_instances[0]
    expect(proc1.name).to eq("new_process1")
    expect(proc1.description).to eq("desc1")

    proc2 = pdb.process_instances[1]
    expect(proc2.name).to eq("new_process2")
    expect(proc2.description).to eq("desc2")
  end

  context "when dsl syntax is correct" do
    it "evaluates correct test files with ProcessDomainBuilder#build_domain" do
      pdb = Stuka::ProcessDomainBuilder.new
      pdb.build_domain
      expect(pdb.process_instances.count).to eq(2)
      expect(pdb.process_instances.map(&:name).include? "check_smth").to be true
      expect(pdb.process_instances.map(&:name).include? "test2").to be true

      expect(pdb.action_instances.count).to eq(2)

      expect(pdb.action_instances[0].name).to eq("acc1")
      expect(pdb.action_instances[0].definition_file).to eq "#{(Stuka::StukaPaths.processes_definition_path)}/correct_test_process.rb"
      expect(pdb.action_instances[0].endings).to eq [:success, :failure]

      expect(pdb.action_instances[1].name).to eq("acc2")
      expect(pdb.action_instances[1].definition_file).to eq "#{(Stuka::StukaPaths.processes_definition_path)}/correct_test_process.rb"
      expect(pdb.action_instances[1].endings).to eq [:success, :failure]
    end
  end

  context "when dsl syntax is incorrect" do
    
    before(:each) do
      Stuka::StukaPaths::PROCESSES_DEFINITION_PATH.replace Stuka::StukaPaths::TEST_PROCESSES_INCORRECT_DEFINITION_PATH
    end

    it "ProcessIncubator#incubate evaluates bad test files" do
      pdb = Stuka::ProcessDomainBuilder.new
      expect{pdb.build_domain}.to raise_error
    end

  end

end