require "spec_helper"
require "stuka/internal/validator"
require 'stuka/internal/domain/process'

describe "Validator" do

  let(:validator) { Stuka::Validator.new }

  let(:valid_process) { Stuka::Process.new("pr1", "pr2_desc", "start_action") }
  let(:valid_process2) { Stuka::Process.new("pr2", "pr2_desc", "start_action") }

  let(:invalid_process) { Stuka::Process.new("pr3", nil, "start_action") }
  let(:invalid_process2) { Stuka::Process.new("", "description", "start_action") }

  context "when testing single process" do

    it "validates process atributes with Validator::process_valid?" do
      expect(validator.process_valid? valid_process).to eq(true)
      expect(validator.process_valid? valid_process2).to eq(true)
      expect(validator.process_valid? invalid_process).to eq(false)
      expect(validator.process_valid? invalid_process2).to eq(false)
    end

  end

  context "when testing a set of processes" do

    let(:valid_process_set) { [valid_process, valid_process2] }
    let(:invalid_process_set) { [valid_process, valid_process, valid_process2] }
    let(:invalid_process_set2) { [valid_process, valid_process, nil] }
    let(:invalid_process_set3) { [valid_process, valid_process2, invalid_process2] }

    it "checks if all processes have different names with Validator::process_set_valid?" do
      expect(validator.process_set_valid? valid_process_set).to eq(true)
      expect(validator.process_set_valid? invalid_process_set).to eq(false)
      expect(validator.process_set_valid? invalid_process_set2).to eq(false)
      expect(validator.process_set_valid? invalid_process_set3).to eq(false)
    end

  end

end